% STDNORMINV_BISECT  Compute the quantile (the inverse of CDF) at a of the
% standard normal distribution using bisection method and normcdf function.
% It is designed as an illustration of the while..end construction usage.
% Warning: it supports only scalar a value.
%
% Usage:
%   x = stdnorminv_bisect(a)

% (C) 2015 Alexey Voskov
% License: public domain
function x = stdnorminv_bisect(a)
if ~isscalar(a) || ~isreal(a) || ~isnumeric(a) || a < 0 || a > 1
  error('a must be a numeric real scalar between 0 and 1');
end
xleft = -20; % Initial left boundary of the interval
xright = 20; % Initial right boundary of the interval
while xright - xleft > eps % eps is machine epsilon (about 2.2e-16 for double)
  % Find center of the inteval and compute function values (at 3 points)
  xcenter = (xleft + xright) / 2;
  fleft = normcdf(xleft) - a;
  fright = normcdf(xright) - a;
  fcenter = normcdf(xcenter) - a;
  % Check if the solution is reached
  if fleft == 0
    xcenter = xleft;
    break;
  elseif fright == 0
    xcenter = xright;
    break;
  elseif fcenter == 0
    break;
  end
  % Check the interval for the next iteration
  if sign(fleft)*sign(fcenter) < 0
    xright = xcenter;
  elseif sign(fcenter)*sign(fright) < 0
    xleft = xcenter;
  else
    error('Internal error');    
  end
end
% Return the result
x = xcenter;
if x == -20
  x = -Inf;
elseif x == 20
  x = +Inf;
end
end
